# This file was created automatically by SWIG.
import switchboardc
class OOFMessagePtr :
    def __init__(self,this):
        self.this = this
        self.thisown = 0
    def name(self, *_args, **_kwargs):
        val = apply(switchboardc.OOFMessage_name,(self,) + _args, _kwargs)
        return val
    def __len__(self, *_args, **_kwargs):
        val = apply(switchboardc.OOFMessage___len__,(self,) + _args, _kwargs)
        return val
    def __getitem__(self, *_args, **_kwargs):
        val = apply(switchboardc.OOFMessage___getitem__,(self,) + _args, _kwargs)
        return val
    def __repr__(self):
        return "<C OOFMessage instance at %s>" % (self.this,)
class OOFMessage(OOFMessagePtr):
    def __init__(self,this):
        self.this = this






#-------------- FUNCTION WRAPPERS ------------------

init_switchboard_api = switchboardc.init_switchboard_api



#-------------- VARIABLE WRAPPERS ------------------



#-------------- USER INCLUDE -----------------------

# -*- python -*-
# $RCSfile: switchboard.spy,v $
# $Revision: 1.1 $
# $Author: langer $
# $Date: 2005/10/18 17:31:10 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 


"""

Class to handle communication between various pieces of OOF.  Classes
can send the switchboard a message when they've done something important,
and can request callbacks when the Switchboard receives a message.

"""

import types
from oof2.common import debug
from oof2.common import thread_enable
from oof2.common import mainthread

class _SwitchboardCallbackBase:
    def __init__(self, msgname, func, caller, args, kwargs):
        self.msgname = msgname
        self.caller = caller            # code line that requested the callback
        self.func = func
        self.args = args
        self.kwargs = kwargs
        self.blocked = 0
    def block(self):
        self.blocked += 1
    def unblock(self):
        self.blocked -= 1
    def is_blocked(self):
        return self.blocked > 0
    def perform(self, *args, **kwargs):
        pass
    def __repr__(self):
        return 'SwitchboardCallbackBase(%s, %s)' % (`self.msgname`, `self.func`)


# Transfer control to the main thread.  This is the default.
class _SwitchboardCallback (_SwitchboardCallbackBase):
    def __init__(self, msgname, func, caller, args, kwargs):
        _SwitchboardCallbackBase.__init__(self, msgname, func, caller,
                                          args, kwargs)
    def perform(self, *args, **kwargs):
        if not self.is_blocked():  # If block is zero or negative, do the call.
            kwargs.update(self.kwargs)      # is this safe?
            if switchboard.verbose:
                debug.fmsg("requested by", self.caller)
            mainthread.runBlock(self.func, self.args + args, kwargs)
            
    def __repr__(self):
        return 'SwitchboardCallback(%s, %s)' % (`self.msgname`, `self.func`)


# Do the callback immediately on the local thread.
class _SwitchboardCallbackThread (_SwitchboardCallbackBase):
    def __init__(self, msgname, func, caller, args, kwargs):
        _SwitchboardCallbackBase.__init__(self, msgname, func, caller,
                                          args, kwargs)
    
    def perform(self, *args, **kwargs):
        if not self.is_blocked():  # If block is zero or negative, do the call.
            kwargs.update(self.kwargs)      # is this safe?
            if switchboard.verbose:
                debug.fmsg("requested by", self.caller)
            self.func(*(self.args + args), **kwargs)
            
    def __repr__(self):
        return 'SwitchboardCallbackThread(%s, %s)' % (`self.msgname`,
                                                      `self.func`)

class _SwitchboardTaskList:
    def __init__(self, msgname):
        self.callbacks = []
        self.msgname = msgname
    def registerCallback(self, func, caller, args, kwargs, child_thread):
        if not child_thread:
            cb = _SwitchboardCallback(self.msgname, func, caller, args, kwargs)
        else:
            cb = _SwitchboardCallbackThread(self.msgname, func, caller,
                                            args, kwargs)
        self.callbacks.append(cb)
        return cb
    def removeCallback(self, callback):
        self.callbacks.remove(callback)
    def perform(self, *args, **kwargs):
        # Callbacks sometimes remove themselves, which modifies the
        # self.callbacks list, so iterate over a copy. TODO: Should
        # the copying process be locked?
        for callback in self.callbacks[:]:
            callback.perform(*args, **kwargs)

    
class _Switchboard:
    def __init__(self):
        self.taskdict = {}
        self.verbose = 0
    def _defineMessage(self, msgname):
        tasklist = _SwitchboardTaskList(msgname)
        self.taskdict[msgname] = tasklist
        return tasklist
    def notify(self, msgname, *args, **kwargs):
        try:
            task = self.taskdict[msgname]
        except KeyError:
            pass
        else:
            if self.verbose:
                debug.fmsg('performing', msgname, args, kwargs)
            task.perform(*args, **kwargs)
            if self.verbose:
                debug.fmsg('done', msgname)

    def cnotify(self, oofmessage):
        # Messages passed from C++ may be either simple strings or
        # OOFMessage instances. The OOFMessage class has swigged
        # methods that make it act like a list of PyObjects.
        if type(oofmessage)==types.StringType:
            self.notify(oofmessage)
        else:
            # Convert OOFMessage object into arglist
            self.notify(oofmessage.name(), *tuple(oofmessage))
    # Message names passed in from Python need not be strings,
    # the only requirement is that they be hashable.  LabelTree,
    # for instance, uses (instance, string) pairs as msgnames.
    def requestCallback(self, msgname, func, caller, *args, **kwargs):
        try:
            tasklist = self.taskdict[msgname]
        except KeyError:
            tasklist = self._defineMessage(msgname)
        except TypeError:
            debug.fmsg("Offending message is", msgname)
            raise
        return tasklist.registerCallback(func, caller,
                                         args, kwargs, child_thread = 0)

    def requestCallbackInThread(self, msgname, func, caller, *args, **kwargs):
        try:
            tasklist = self.taskdict[msgname]
        except KeyError:
            tasklist = self._defineMessage(msgname)
        return tasklist.registerCallback(func, caller,
                                         args, kwargs, child_thread = 1)
    

    
    # Removes a callback from the tasklist, and removes the tasklist
    # from the taskdict if its list has become empty.
    def removeCallback(self, action):
        task = self.taskdict[action.msgname]
        task.removeCallback(action)
        if len(task.callbacks)==0:
            del self.taskdict[action.msgname]
    # Switchboard entries for block and unblock -- in most cases,
    # it's probably clearer to just locally block and unblock
    # the SwitchboardCallback (returned from the requestCallback)
    # directly, but these entry points are available for full
    # generality.
    #   Should catch KeyError and do something sensible?
    def blockCallback(self, action):
        self.taskdict[action.msgname].block()
    def unblockCallback(self, action):
        self.taskdict[action.msgname].unblock()
    


switchboard = _Switchboard()
init_switchboard_api(switchboard.cnotify)

def requestCallback(msgname, func, *args, **kwargs):
    caller = debug.callerID()
    return switchboard.requestCallback(msgname, func, caller, *args, **kwargs)

def notify(msgname, *args, **kwargs):
    if switchboard.verbose:
        debug.dumpCaller()
    switchboard.notify(msgname, *args, **kwargs)
    
def removeCallback(action):
    switchboard.removeCallback(action)

def requestCallbackInThread(msgname, func, *args, **kwargs):
    caller = debug.callerID()
    return switchboard.requestCallbackInThread(msgname, func, caller,
                                               *args, **kwargs)

def verbose(menuitem, flag):
    switchboard.verbose = flag
