# -*- python -*-
# $RCSfile: oofGUI.py,v $
# $Revision: 1.45 $
# $Author: langer $
# $Date: 2005/08/12 19:17:46 $


# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 


from oof2.common import debug
from oof2.common import guitop
from oof2.common import mainthread
from oof2.common import thread_enable
from oof2.common import utils
from oof2.common.IO import gfxmanager
from oof2.common.IO import mainmenu
from oof2.common.IO import oofmenu
from oof2.common.IO.GUI import chooser
from oof2.common.IO.GUI import gfxmenu
from oof2.common.IO.GUI import quit
from oof2.common.IO.GUI import widgetscope
from oof2.common.IO.GUI import gtkutils
from oof2.common.IO import reporter
import oof2.common.quit
import gtk
import pango

allPages = {}                           # dictionary of pages keyed by name
pagenames = []                          # ordered list of pages

initial_width = 550
initial_height = 350

class oofGUI(widgetscope.WidgetScope):
    def __init__(self):
        debug.mainthreadTest()
        widgetscope.WidgetScope.__init__(self, None)
        self.gtk = gtk.Window(gtk.WINDOW_TOPLEVEL)

##        settings = gtk.settings_get_default()
##        settings.set_property("gtk-font-name", "Sans 12")

        self.gtk.set_title('OOF2')
        self.gtk.set_default_size(initial_width, initial_height)
        self.gtk.connect("destroy", self.destroyCB)
        self.gtk.connect("delete-event", self.deleteEventCB)
        guitop.setTop(self)
        self.tooltips = gtk.Tooltips()

        self.mainbox = gtk.VBox()
        self.gtk.add(self.mainbox)

        self.menubar = gtk.MenuBar()
        self.mainbox.pack_start(self.menubar, expand=0, fill=0)
        accelgrp = gtk.AccelGroup()
        self.gtk.add_accel_group(accelgrp)

        self.mainmenu = mainmenu.OOF
        self.oofmenu = gfxmenu.gtkOOFMenuBar(self.mainmenu, bar=self.menubar,
                                             accelgroup=accelgrp)
        self.pageChooserFrame = gtk.Frame()
        self.pageChooserFrame.set_shadow_type(gtk.SHADOW_IN)
        self.mainbox.pack_start(self.pageChooserFrame,
                                expand=0, fill=0, padding=2)

        align = gtk.Alignment(xalign=0.5)
        self.pageChooserFrame.add(align)
        chooserBox = gtk.HBox()
        chooserBox.set_border_width(2)
        align.add(chooserBox)

        label = gtk.Label('Task: ')
        label.set_alignment(1.0, 0.5)
        chooserBox.pack_start(label, expand=0, fill=0)
        self.prevPageButton = gtk.Button()
        chooserBox.pack_start(self.prevPageButton, expand=0, fill=0,
                              padding=3)
        self.prevPageButton.add(gtk.Arrow(gtk.ARROW_LEFT, gtk.SHADOW_OUT))
        self.prevPageButton.connect('clicked', self.prevPageCB)
        self.pageChooser = chooser.ChooserWidget([],
                                                 callback=self.pageChooserCB)
        chooserBox.pack_start(self.pageChooser.gtk, expand=0, fill=0)
        self.currentPageName = None

        self.nextPageButton = gtk.Button()
        chooserBox.pack_start(self.nextPageButton, expand=0, fill=0,
                              padding=3)
        self.nextPageButton.add(gtk.Arrow(gtk.ARROW_RIGHT, gtk.SHADOW_OUT))
        self.nextPageButton.connect('clicked', self.nextPageCB)


        # Find the font size, so widgets can be sized appropriately.
        #  digitsize and charsize are in pixels.
        
        self.digitsize,self.charsize = gtkutils.widgetFontSizes(self.gtk)

        # Add a GUI callback to the "OOF2" windows item.
        oof2_item = self.mainmenu.Windows.OOF2
        oof2_item.add_gui_callback(self.menu_raise)

        # Frame around main pages.  GUI pages are added and removed
        # from it by installPage().
        self.pageframe = gtk.Frame()
##        self.pageframe.set_size_request(MainPage.initial_width,
##                                        MainPage.initial_height)
        self.pageframe.set_shadow_type(gtk.SHADOW_IN)
        self.mainbox.pack_start(self.pageframe, expand=1, fill=1)

        # Add pages that may have been created before the main GUI was built.
        for pagename, i in zip(pagenames, range(len(allPages))):
            self.addPage(allPages[pagename], i)

    def tooltip(self, widget, tip):
        debug.mainthreadTest()
        self.tooltips.set_tip(widget, tip)

    def installPage(self, pagename):
        # Actually install a page in the page frame.
        if self.currentPageName is not None:
            self.pageframe.remove(allPages[self.currentPageName].gtk)
        self.currentPageName = pagename
        self.pageframe.add(allPages[self.currentPageName].gtk)
        self.sensitize()

    def show(self, messages=[]):
        debug.mainthreadTest()
        if self.currentPageName is None:
            self.installPage(pagenames[0])
            self.pageChooser.set_state(self.currentPageName)

        # don't use self.gtk.show_all(), since there may be page
        # components that shouldn't yet be shown.
        self.menubar.show_all()
        self.pageChooserFrame.show_all()
        self.pageframe.show()
        self.mainbox.show()
        for page in allPages.values():
            page.show()
        self.gtk.show()

        for m in messages:
            reporter.report(m)
        
    def addPage(self, page, position):
        pagetips = {}
        for pg in allPages.values():
            if pg.tip:
                pagetips[pg.name]=pg.tip
        self.pageChooser.update(pagenames, pagetips)
        self.sensitize()

    def pageChooserCB(self, widget, pagename):
        self.installPage(pagename)

    def nextPageCB(self, button):
        which = pagenames.index(self.currentPageName)
        self.installPage(pagenames[which+1])
        self.pageChooser.set_state(self.currentPageName)

    def prevPageCB(self, button):
        which = pagenames.index(self.currentPageName)
        self.installPage(pagenames[which-1])        
        self.pageChooser.set_state(self.currentPageName)

    def sensitize(self):
        # If currentPageName is None, then the GUI hasn't been shown yet.
        if self.currentPageName is not None:
            which = pagenames.index(self.currentPageName)
            self.nextPageButton.set_sensitive(which != len(pagenames)-1)
            self.prevPageButton.set_sensitive(which != 0)

            if which < len(pagenames)-1:
                self.tooltip(self.nextPageButton, "Go to the %s page" %
                             allPages[pagenames[which+1]].name)
            if which > 0:
                self.tooltip(self.prevPageButton, "Go to the %s page" %
                             allPages[pagenames[which-1]].name)

    def deleteEventCB(self, *args):
        if quit.quitQuery():            # do you really want to quit?
            oof2.common.quit.cleanup()
            return 0                    # event not handled, will call destroyCB
        return 1                        # don't actually destroy the window
    def destroyCB(self, gtk):
        self.gtk = None
        guitop.setTop(None)
    def destroy(self):
        debug.mainthreadTest()
        if self.gtk:
            self.gtk.destroy()

    def stop(self):
        # Close all windows
        from oof2.common.IO.GUI import layereditorGUI
        layereditorGUI.destroyLayerEditor()
        for window in gfxmanager.gfxManager.getAllWindows():
            gfxmanager.gfxManager.closeWindow(window)
        
        gtk.main_quit()

    # GUI callback for "OOF2" entry in Windows menu.  Normally,
    # the "subwindow" class provides this functionality, but the main
    # window is not a subwindow.
    def menu_raise(self, menuitem):
        debug.mainthreadTest()
        self.gtk.window.raise_()
        

    
####################################################

# Pages added to the main OOF window must be subclasses of MainPage.
# The subclass's __init__ should call MainPage.__init__ and install
# all of its widgets into self.gtk, which is a GtkFrame constructed by
# MainPage.__init__.  A single instance of the subclass should be
# created.

class MainPage(widgetscope.WidgetScope):
    # TODO LATER: Set initial sizes in some more user-friendly way.
    def __init__(self, name, ordering, tip=None):
        debug.mainthreadTest()
        widgetscope.WidgetScope.__init__(self, parent=gui)
        self.name = name
        self.ordering = ordering
        self.tip = tip
        self.gtk = gtk.Frame()
        self.gtk.set_shadow_type(gtk.SHADOW_NONE)
        # Insert the page in the proper spot
        for i in range(len(pagenames)):
            if self.ordering < allPages[pagenames[i]].ordering:
                pagenames.insert(i, name)
                if guitop.top():        # gui may not be constructed yet!
                    guitop.top().addPage(self, i)
                break
        else:                           # page goes at the end
            pagenames.append(name)
            if guitop.top():            # gui may not be constructed yet!
                guitop.top().addPage(self, len(pagenames))
        allPages[name] = self

    # MainPage.show() should be redefined in subclasses that don't
    # always want to show all their widgets.
    def show(self):
        self.gtk.show_all()
    def hide(self):
        self.gtk.hide()

gui = oofGUI()

def start(messages=[]):
    mainthread.rethread() # Make this thread "main".
    debug.mainthreadTest()
    gui.show(messages)
    guitop.setMainLoop(1)
    if thread_enable.enabled():
        gtk.threads_init()
        gtk.threads_enter()
    gtk.main()
    if thread_enable.enabled():
        gtk.threads_leave()
    guitop.setMainLoop(0)

