// -*- C++ -*-
// $RCSfile: coord.C,v $
// $Revision: 1.9 $
// $Author: langer $
// $Date: 2005/09/20 20:17:30 $

/* This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance. However, to
 * facilitate maintenance we ask that before distributing modified
 * versions of this software, you first contact the authors at
 * oof_manager@ctcms.nist.gov. 
 */

#include <oofconfig.h>
#include "coord.h"

Coord &Coord::operator+=(const ICoord &ic) {
  x[0] += ic(0);
  x[1] += ic(1);
  return *this;
}

Coord &Coord::operator-=(const ICoord &ic) {
  x[0] -= ic(0);
  x[1] -= ic(1);
  return *this;
}

std::ostream &operator<<(std::ostream &os, const Coord &coord) {
  os << "(" << coord(0) << ", " << coord(1) << ")";
  return os;
}

std::istream &operator>>(std::istream &is, Coord &coord) {
  char c;
  is >> c;
  if(!is || c != '(') {		// check for initial '('
    is.clear(std::ios::badbit | is.rdstate());
    return is;
  }
  is >> coord(0);		// read x component
  if(!is) return is;		// check for error
  is >> c;
  if(!is || c != ',') {		// check for ','
    is.clear(std::ios::badbit | is.rdstate());
    return is;
  }
  is >> coord(1);
  if(!is) return is;
  is >> c;
  if(!is || c != ')') {
    is.clear(std::ios::badbit | is.rdstate());
    return is;
  }
  return is;
}

//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//=\\=//

std::ostream &operator<<(std::ostream &os, const ICoord &coord) {
  os << "(" << coord(0) << ", " << coord(1) << ")";
  return os;
}

std::istream &operator>>(std::istream &is, ICoord &coord) {
  char c;
  is >> c;
  if(!is || c != '(') {		// check for initial '('
    is.clear(std::ios::badbit | is.rdstate());
    return is;
  }
  is >> coord(0);		// read x component
  if(!is) return is;		// check for error
  is >> c;
  if(!is || c != ',') {		// check for ','
    is.clear(std::ios::badbit | is.rdstate());
    return is;
  }
  is >> coord(1);
  if(!is) return is;
  is >> c;
  if(!is || c != ')') {
    is.clear(std::ios::badbit | is.rdstate());
    return is;
  }
  return is;
}

bool operator<(const ICoord &a, const ICoord &b) {
  if(a(0) < b(0)) return true;
  if(a(0) > b(0)) return false;
  if(a(1) < b(1)) return true;
  return false;
}


//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//

// TODO: Inline this function?
double CoordSegment::area(const Coord &ctr) const {
  // Avoid constructing new objects...
  return (start(0)-ctr(0))*(end(1)-ctr(1)) - (start(1)-ctr(1))*(end(0)-ctr(0));
}

bool CoordSegment::operator<(const CoordSegment &other) const {
  if (other.start < start) return true;
  if (start < other.start) return false;
  if (other.end < end) return true;
  return false;
}
