// -*- C++ -*-
// $RCSfile: mpitools.h,v $
// $Revision: 1.15 $
// $Author: redwing $
// $Date: 2004/07/08 19:04:51 $

/* This software was produced by NIST, an agency of the U.S. government,
 * and by statute is not subject to copyright in the United States.
 * Recipients of this software assume all responsibilities associated
 * with its operation, modification and maintenance. However, to
 * facilitate maintenance we ask that before distributing modified
 * versions of this software, you first contact the authors at
 * oof_manager@ctcms.nist.gov. 
 */

#include <oofconfig.h>

#ifndef MPITOOLS_H
#define MPITOOLS_H

#include <Python.h>
#include <vector>
#include "ooferror.h"
#include <string>
#include <mpi.h>

/* Initialization routines */
void _mpi_initialize(std::vector<char*>*);
int mpi_initialized();
void mpi_finalize();
int mpi_abort();
/* End of initializations routines */


/* processes properties routines*/
int size();
int rank();
std::string get_processor_name();

/* communicator routines */




// point to point communication functions and classes



int send_string(const std::string &, int);
std::string receive_string(int);

int send_int(int &, int);
int receive_int(int);

int send_vector_of_ints(std::vector<int> *, int); //arguments:(message, destination)
std::vector<int> * receive_vector_of_ints(int); //argument: (origin)

int send_vector_of_doubles(std::vector<double> *, int); //arguments:(message, destination)
std::vector<double> * receive_vector_of_doubles(int); //argument: (origin)



//collective communication functions
int broadcast_string(const std::string &, int); //arguments: (message, origin)
std::string receive_broadcast_string( int); //argument: (origin)
 
int broadcast_int(int &, int); //arguments: (message, origin)
int receive_broadcast_int(int); //argument: (origin)

int broadcast_double(double&, int);//arguments: (message, origin)
double receive_broadcast_double(int);//argument: (origin)

int broadcast_vector_of_shorts(std::vector<unsigned short> *, int); //arguments: (message, origin)
std::vector<unsigned short> * receive_broadcast_vector_of_shorts(int); //argument: (origin)

int broadcast_vector_of_ints(std::vector<int> *, int); //arguments: (message, origin)
std::vector<int> * receive_broadcast_vector_of_ints(int); //argument: (origin)

int broadcast_vector_of_doubles(std::vector<double> *, int); //arguments: (message, origin)
std::vector<double> * receive_broadcast_vector_of_doubles(int); //argument: (origin)

/* blocking routines */
int mpi_barrier();


class MPICommunicator {
private:
  MPI_Comm communicator;
public:
  MPICommunicator();
  ~MPICommunicator(){};
  MPI_Comm & get() {return communicator;}
};

//class to throw/catch MPI Exceptions
class MPIException : public ErrError {
private:
  const std::string file;
  const int line;
  const std::string msg;
  const int process;
public:
  MPIException(const std::string &m, const std::string &f, int l, int proc_num);
  virtual ~MPIException() {}
  virtual std::string message() const {return msg;};
  std::string filename() const { return file; } 
  int lineno() const { return line; }           
  virtual std::string pythonequiv() const;
};


MPICommunicator & getOOF_Communicator();

#endif // MPITOOLS_H




