# -*- python -*-
# $RCSfile: solverPage.py,v $
# $Revision: 1.60 $
# $Author: reida $
# $Date: 2006/01/23 22:05:53 $


# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from oof2.SWIG.common import devel
from oof2.SWIG.common import switchboard
from oof2.common import debug
from oof2.common import labeltree
from oof2.common import microstructure
from oof2.common import utils
from oof2.common import guitop
from oof2.common.IO import mainmenu
from oof2.common.IO import parameter, reporter
from oof2.common.IO import whoville
from oof2.common.IO.GUI import chooser
from oof2.common.IO.GUI import oofGUI
from oof2.common.IO.GUI import regclassfactory
from oof2.common.IO.GUI import whowidget
from oof2.engine.IO import subproblemmenu

import string

## Import of different wrapped solvers
from oof2.SWIG.engine import solver
try:
    from oof2.SWIG.engine.PETSc import petsc_solver
except ImportError:
    pass

from oof2.SWIG.engine import solverdriver
## Import of different wrapped preconditioners
from oof2.SWIG.engine import preconditioner
try:
    from oof2.SWIG.engine.PETSc import petsc_preconditioner
except ImportError:
        pass

from oof2.engine.IO import solvermenu
from oof2.engine import skeletoncontext
from oof2.engine import timedrivers
import gtk
import oof2.engine.mesh

OOF = mainmenu.OOF

class SolverSelectFactory(regclassfactory.RegisteredClassFactory):
    def __init__(self, registry, obj=None, title=None,
                 callback=None, fill=0, expand=0, scope=None,
                 widgetdict={}, *args, **kwargs):
        self.meshwidget = scope.findWidget(
            lambda w: isinstance(w, whowidget.WhoWidget)
            and w.whoclass is oof2.engine.mesh.meshes)
        self.sbcb = switchboard.requestCallback(self.meshwidget, self.mwCB)
        self.meshobj = None

        regclassfactory.RegisteredClassFactory.__init__(
            self, registry, obj=obj, title=title, callback=callback,
            fill=fill, expand=expand, scope=scope, widgetdict=widgetdict,
            *args, **kwargs)

    def cleanUp(self):
        switchboard.removeCallback(self.sbcb)
        regclassfactory.RegisteredClassFactory.cleanUp(self)

    def mwCB(self, *args, **kwargs):
        meshname = self.meshwidget.get_value()
        # Tests if you got a nontrival fully qualified mesh name.
        if string.split(meshname,':')[2]:
            self.meshobj = oof2.engine.mesh.meshes[meshname]
        else:
            self.meshobj = None

    def refresh(self):
        self.update(self.registry)
            
    def includeRegistration(self, reg):
        # If the matrix is not symmetric, then exclude the solvers
        # which *are* symmetric, otherwise, just include all
        # non-secret solvers.
        if self.meshobj:
            if self.meshobj.get_symmetry_state() != \
                   oof2.engine.mesh.SYM_STATE_SYMMETRIC:
                return (not reg.symmetric) and (not reg.secret)
            else:
                return not reg.secret
        else:
            return not reg.secret
            
def _makeLinearSolverWidget(self, scope=None):
    return SolverSelectFactory(self.registry, scope=scope)

solver.LinearSolverParameter.makeWidget = _makeLinearSolverWidget
            

class SolverPage(oofGUI.MainPage):
    def __init__(self):
        self.sbcallbacks = [
            switchboard.requestCallback("mesh changed", self.meshChanged),
            switchboard.requestCallback("mesh solved",
                                        self.meshSolved),
            switchboard.requestCallback("made reservation",
                                        self.reservationChanged),
            switchboard.requestCallback("cancelled reservation",
                                        self.reservationChanged),
            switchboard.requestCallback(("new who", "Skeleton"),
                                        self.newSkeleton),
            switchboard.requestCallback(("new who", "Mesh"), self.newMesh),
            switchboard.requestCallback(("remove who", "Mesh"), self.removeMesh)
            ]
            
        oofGUI.MainPage.__init__(self, name="Solver",
                                 ordering=240,
                                 tip="Set mesh and time driver to find the solution of active fields and partial differential equations")
        mainbox = gtk.VBox()
        self.gtk.add(mainbox)

        align = gtk.Alignment(xalign=0.5)
        mainbox.pack_start(align, expand=0, fill=0)
        centerbox = gtk.HBox(spacing=3)
        align.add(centerbox)
        if devel.devel():
            self.meshwidget = whowidget.WhoWidget(
                whoville.getClass('SubProblemWho'),
                callback=self.meshCB, scope=self)
        else:
            self.meshwidget = whowidget.WhoWidget(
                oof2.engine.mesh.meshes,
                callback=self.meshCB, scope=self)
        label = gtk.Label("Microstructure=")
        label.set_alignment(1.0, 0.5)
        centerbox.pack_start(label, expand=0, fill=0)
        centerbox.pack_start(self.meshwidget.gtk[0], expand=0, fill=0)
        label = gtk.Label("Skeleton=")
        label.set_alignment(1.0, 0.5)
        centerbox.pack_start(label, expand=0, fill=0)
        centerbox.pack_start(self.meshwidget.gtk[1], expand=0, fill=0)
        label = gtk.Label("Mesh=")
        label.set_alignment(1.0, 0.5)
        centerbox.pack_start(label, expand=0, fill=0)
        centerbox.pack_start(self.meshwidget.gtk[2], expand=0, fill=0)
        if devel.devel():
            label = gtk.Label("SubProblem=")
            label.set_alignment(1.0, 0.5)
            centerbox.pack_start(label, expand=0, fill=0)
            centerbox.pack_start(self.meshwidget.gtk[3], expand=0, fill=0)
        
        panes = gtk.HPaned()
        mainbox.pack_start(panes, expand=1, fill=1)

        statusframe = gtk.Frame("Mesh Status")
        statusframe.set_shadow_type(gtk.SHADOW_IN)
        panes.pack1(statusframe, resize=True)
        scroll = gtk.ScrolledWindow()
        scroll.set_shadow_type(gtk.SHADOW_IN)
        scroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        statusframe.add(scroll)
        self.statustext = gtk.TextView()
        self.statustext.set_wrap_mode(gtk.WRAP_WORD)
        self.statustext.set_editable(False)
        self.statustext.set_cursor_visible(False)
        scroll.add(self.statustext)

        ## solver component selection
        solverVwindow = gtk.VBox()
        panes.pack2(solverVwindow, resize=0, shrink=0)
        ## solver driver selection
        if devel.devel():
            solverdriverframe = gtk.Frame("MatrixDriver")
        else:
            solverdriverframe = gtk.Frame("Solver")
        solverdriverframe.set_shadow_type(gtk.SHADOW_IN)
        ## panes.pack2(solverdriverframe, shrink=0)
        solverVwindow.pack_start(solverdriverframe, expand=1, fill=1)
        self.driverSelectorFactory = regclassfactory.RegisteredClassFactory(
            solverdriver.Driver.registry, scope=self)
        solverdriverframe.add(self.driverSelectorFactory.gtk)

        # Extract the solver widget, so it can be updated.
        self.solverwidget = self.driverSelectorFactory.findWidget(
            lambda w: isinstance(w, SolverSelectFactory) )
        
        if devel.devel():
            ## time_driver selection
            self.time_driverSelectorFactory = regclassfactory.RegisteredClassFactory(
                timedrivers.TimeDriver.registry)
            time_driverframe = gtk.Frame("TimeDriver")
            time_driverframe.set_shadow_type(gtk.SHADOW_IN)
            solverVwindow.pack_start(time_driverframe, expand=1, fill=1)
            time_driverframe.add(self.time_driverSelectorFactory.gtk)
        
        solverbuttons = gtk.HBox(spacing=2, homogeneous=1)
        solverVwindow.pack_start(solverbuttons, expand=0, fill=0)
        if devel.devel():
            self.resetButton = gtk.Button("Reset")
            self.resetButton.connect("clicked", self.resetCB)
            guitop.top().tooltip(self.resetButton,
                        "set fields back to their (unsolved) initial state")
            solverbuttons.pack_start(self.resetButton, expand=1, fill=1)
            self.setButton = gtk.Button("Set")
            self.setButton.connect("clicked", self.setCB)
            guitop.top().tooltip(self.setButton, "Assign solver to subproblem")
            solverbuttons.pack_start(self.setButton, expand=1, fill=1)
        self.solveButton = gtk.Button("Solve")
        self.solveButton.connect("clicked", self.solveCB)
        guitop.top().tooltip(self.solveButton, "Apply solver to finite element mesh and associated equations and fields")
        solverbuttons.pack_start(self.solveButton, expand=1, fill=1)
        
        self.sensitize()
    def setCB(self, gtkobj):
        solver_selection = self.driverSelectorFactory.get_value()
        time_driver_selection = self.time_driverSelectorFactory.get_value()
        subproblemmenu.solvermenu.SetSolver(subproblem = self.currentFullSubProblemName(),
                                 matrixdriver=solver_selection,
                                 timedriver=time_driver_selection)



        
    def show(self):
        debug.mainthreadTest()
        self.gtk.show_all()
        self.update()

    #######################

    def currentMSName(self):
        path = labeltree.makePath(self.currentFullMeshName())
        return path[0]
    def currentMS(self):
        msname = self.currentMSName()
        if msname:
            return microstructure.microStructures[msname].getObject()

    def currentSkeletonName(self):
        path = labeltree.makePath(self.currentFullMeshName())
        return path[1]
    def currentSkeleton(self):
        ms = self.currentMS()
        skelname = self.currentSkeletonName()
        if skelname:
            return skeletoncontext.getSkeleton(ms, skelname)

    def currentFullMeshName(self):
        if devel.devel():
            pname = self.meshwidget.get_value()
            path = labeltree.makePath(pname)
            return path[0] + ':' + path[1] + ':' + path[2]
        else:
            return self.meshwidget.get_value()

    def currentFullSubProblemName(self):
        return self.meshwidget.get_value()
    
    def currentMeshName(self):
        path = labeltree.makePath(self.currentFullMeshName())
        return path[2]
    def currentMeshContext(self):
        try:
            return oof2.engine.mesh.meshes[self.currentFullMeshName()]
        except KeyError:
            return None
    def currentMesh(self):
        meshcontext = self.currentMeshContext()
        if meshcontext is not None:
            return meshcontext.getObject()

    def sensitize(self):
        debug.mainthreadTest()
        msok = self.currentMS() is not None
        skelok = msok and self.currentSkeleton() is not None
        mesh = self.currentMesh()
        meshcontext = self.currentMeshContext()
        meshok = skelok and (mesh is not None)
        self.solveButton.set_sensitive(
            meshok and                  # we have a mesh
            not meshcontext.query_reservation() and # ... and it's not busy
            meshcontext.materialsConsistent() # ... and it's computable
            )
        if devel.devel():
            self.resetButton.set_sensitive(
                meshok and                  # we have a mesh
                not meshcontext.query_reservation() # ... and it's not busy
                )
            
    def update(self):
        self.set_state(self.currentFullMeshName())
        self.updateStatus()

    def updateStatus(self):
        meshcontext = self.currentMeshContext()
        if meshcontext is not None:
            self.statustext.get_buffer().set_text(meshcontext.status())
        else:
            self.statustext.get_buffer().set_text("No mesh!")
        self.solverwidget.refresh()
        
    def set_state(self, meshpath):
        path = labeltree.makePath(meshpath)
        self.meshwidget.set_value(path)
        mesh = self.currentMesh()
        self.updateStatus()
        

    def newSkeleton(self, skeletonname): # switchboard ("new who", "Skeleton")
        self.sensitize()

    def newMesh(self, meshname):        # switchboard ("new who", "Mesh")
        self.set_state(meshname)
        self.sensitize()

    def meshCB(self, meshpath):         # WhoWidget callback
        self.set_state(meshpath)
        self.sensitize()

    def removeMesh(self, path):         # switchboard ("remove who", "Mesh")
        self.update()
        self.sensitize()

    def reservationChanged(self, whoobj):
        # switchboard "made reservation", "cancelled reservation"
        if  self.currentFullMeshName() == whoobj.path():
            self.sensitize()

    def meshChanged(self, meshcontext):
        if self.currentMeshContext() is meshcontext:
            self.sensitize()
            self.updateStatus()

    def meshSolved(self, meshcontext):
        if self.currentMeshContext() is meshcontext:
            self.sensitize()
            self.updateStatus()

    ###############################

    def solveCB(self, gtkobj):    # "Solve" button callback.
        the_mesh = self.currentFullMeshName()
        subproblem_selection = self.currentFullSubProblemName()
        solver_selection = self.driverSelectorFactory.get_value()
        if devel.devel():
            time_driver_selection = self.time_driverSelectorFactory.get_value()
            ## OOF.Solver.Solve(mesh=the_mesh, matrixdriver=solver_selection, timedriver=time_driver_selection)
            subproblemmenu.solvermenu.Solve(subproblem = subproblem_selection,
                                            matrixdriver=solver_selection,
                                            timedriver=time_driver_selection)
        else:
            OOF.Solver.Solve(mesh=the_mesh, solver=solver_selection)
            
    def resetCB(self, gtkobj):
        if devel.devel():
            subproblem_selection = self.currentFullSubProblemName()
            subproblemmenu.solvermenu.Reset(subproblem = subproblem_selection)
        else:
            the_mesh = self.currentFullMeshName()
            OOF.Solver.Reset(mesh=the_mesh)
        
#####################################################################

solverpage = SolverPage()



