# -*- python -*-
# $RCSfile: subproblemPage.py,v $
# $Revision: 1.20 $
# $Author: reida $
# $Date: 2006/01/10 22:05:38 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from oof2.SWIG.common import switchboard
from oof2.common import debug
from oof2.common import guitop
from oof2.common import labeltree
from oof2.common import microstructure
from oof2.common.IO import mainmenu
from oof2.common.IO import parameter
from oof2.common.IO import whoville
from oof2.common.IO.GUI import chooser
from oof2.common.IO.GUI import oofGUI
from oof2.common.IO.GUI import parameterwidgets
from oof2.common.IO.GUI import regclassfactory
from oof2.common.IO.GUI import whowidget
from oof2.engine import fieldinit
from oof2.engine import skeletoncontext
from oof2.engine.IO import subproblemmenu
import gtk
import oof2.engine.mesh
import oof2.SWIG.engine.field
import string
from oof2.SWIG.engine import solverdriver
from oof2.engine import timedrivers

allFields = oof2.SWIG.engine.field.allFields

meshmenu = mainmenu.OOF.Mesh

class ButtonSignal:
    def __init__(self, button, signal):
        self.button = button
        self.signal = signal
    def set(self, active):
        debug.mainthreadTest()
        self.button.signal_handler_block(self.signal)
        self.button.set_active(active)
        self.button.signal_handler_unblock(self.signal)
    def set_sensitive(self, sensitivity):
        debug.mainthreadTest()
        self.button.set_sensitive(sensitivity)

class SubPage:
    def __init__(self, mainpage):
        self.mainpage = mainpage
    def add_widgets_to_frame(self, frame):
        ## may also add switchboard callbacks
        pass ## re-written in subclasses
class ActiveRegionSubPage (SubPage):
    def __init__(self, mainpage):
        SubPage.__init__(self, mainpage)
        switchboard.requestCallback("groupset member added",
                                    self.draw_grouplist),
        switchboard.requestCallback("groupset member renamed",
                                    self.draw_grouplist),
        switchboard.requestCallback("groupset changed",
                                    self.draw_grouplist),
        switchboard.requestCallback("groupset member resized",
                                    self.draw_grouplist)
     
    def add_widgets_to_frame(self, frame):
        self.vWindow = gtk.VBox()
        frame.add(self.vWindow)
        self.grouplist = chooser.ScrolledChooserListWidget(
            callback=self.chooserCB)
        self.vWindow.pack_start(self.grouplist.gtk, fill=1, expand=1)
        self.setButton = gtk.Button("Set")
        self.setButton.connect("clicked", self.setCB)
        self.setButton.set_sensitive(0)
        self.vWindow.pack_start(self.setButton, fill=0, expand=0)
        self.draw_grouplist()
    def chooserCB(self, name):
        pass
    def setCB(self, gtkobj):
        gname = self.grouplist.get_value()
        pnamesel = self.mainpage.meshwidget.get_value()
        meshname = self.mainpage.currentFullMeshName()
        skelpath = ":".join(labeltree.makePath(meshname)[:2])
        
        subproblemmenu.subproblemMenu.SetRegion(subproblem = pnamesel,
                                                skeleton = skelpath, 
                                                group = gname)
    def getGroupSet(self):
        skeletoncontext = self.mainpage.currentSkeleton()
        if skeletoncontext is not None:
            return skeletoncontext.elementgroups
        return None
    
    def draw_grouplist(self, skelcontext = None, gset = None, name = None):
        groupset = self.getGroupSet()
        if groupset is not None:
            namelist = groupset.allGroups()
            self.grouplist.update(namelist,
                                  map(groupset.displayString, namelist))
        else:
            self.grouplist.update([],[])

        
class SubProblemPage(oofGUI.MainPage):
    def __init__(self):
        oofGUI.MainPage.__init__(self, name="SubProblems", ordering=205,
                                 tip="Define SubProblem region to solve for within a Mesh")
        self.last_selection = None ## current sproblem name
        mainbox = gtk.VBox(spacing=2)
        self.gtk.add(mainbox)
        ## main box
        align = gtk.Alignment(xalign=0.5)
        mainbox.pack_start(align, expand=0, fill=0)
        ## mesh selection widget
        centerbox = gtk.HBox(spacing=3)
        align.add(centerbox)

        self.meshwidget = whowidget.WhoWidget(whoville.getClass('SubProblemWho'),
                                              callback=self.meshCB, scope=self)
        label = gtk.Label("Microstructure=")
        label.set_alignment(1.0, 0.5)
        centerbox.pack_start(label, expand=0, fill=0)
        centerbox.pack_start(self.meshwidget.gtk[0], expand=0, fill=0)
        label = gtk.Label("Skeleton=")
        label.set_alignment(1.0, 0.5)
        centerbox.pack_start(label, expand=0, fill=0)
        centerbox.pack_start(self.meshwidget.gtk[1], expand=0, fill=0)
        label = gtk.Label("Mesh=")
        label.set_alignment(1.0, 0.5)
        centerbox.pack_start(label, expand=0, fill=0)
        centerbox.pack_start(self.meshwidget.gtk[2], expand=0, fill=0)
        label = gtk.Label("SubProblem=")
        label.set_alignment(1.0, 0.5)
        centerbox.pack_start(label, expand=0, fill=0)
        centerbox.pack_start(self.meshwidget.gtk[3], expand=0, fill=0)
        ## subproblem creation/selection target
        subpcenterbox = gtk.HBox(homogeneous = 1, spacing=3)
        mainbox.pack_start(subpcenterbox, expand=1, fill=1)
        
        ## new subproblem
        buttonbox = gtk.HBox(homogeneous=1, spacing=3)
        mainbox.pack_start(buttonbox, padding=50, expand=1, fill=1)
        self.newsubproblem = gtk.Button('New...')
        self.newsubproblem.connect('clicked', self.new_subproblem_CB)
        buttonbox.pack_start(self.newsubproblem, expand=1, fill=1)
        guitop.top().tooltip(self.newsubproblem, 'Create a new subproblem')
        self.newsubproblem.set_sensitive(0)

        
        ## notebook creation
        self.notebook = gtk.Notebook()
        mainbox.pack_start(self.notebook, expand=1, fill=1, padding=3)
        self.notebook.show()
        ## ACTIVE REGION
        self.regionframe = gtk.Frame("Region")
        regionlabel = gtk.Label("Region")
        self.regionpage = ActiveRegionSubPage(self)
        self.add_page_widgets(self.regionframe, self.regionpage)
        self.notebook.append_page(self.regionframe, regionlabel)
        ## SOLVER PAGE
        ## self.solverframe = gtk.Frame("Solver")
        ## solverlabel = gtk.Label("Solver")
        ## self.add_page_widgets(self.solverframe, page = SolverSubPage(self))
        ## self.notebook.append_page(self.solverframe, solverlabel)
        
        ## COMMON SWITCHBOARD CALL DEFINITIONS
        switchboard.requestCallback("new subproblem",self.new_sub)
        switchboard.requestCallback(("new who", "Mesh"), self.newMesh)
        switchboard.requestCallback(("remove who", "Mesh"), self.removeMesh)

    def new_subproblem_CB(self, gtkobj):
        paramset = filter(lambda x: x.name!='mesh',
                          subproblemmenu.subproblemMenu.New.params)
        if parameterwidgets.getParameters(title='New SubProblem', *paramset):
            subproblemmenu.subproblemMenu.New.callWithDefaults(
                mesh=self.currentFullMeshName())
        self.sensitize()

    def new_sub(self, name, mesh):
        meshcontext = oof2.engine.mesh.meshes[mesh]
        # Select it.  It was appended, so it's the last item.
        prob_dict = meshcontext.subproblems
        plist = prob_dict.keys()
        ## self.problemname.update(plist)
        ## self.problemname.set_state(name)

    def add_page_widgets(self, frame, page):
        page.add_widgets_to_frame(frame)
        
    def current_subproblems(self): ## returns a list of subproblem names
        plist = []
        mesh=self.currentFullMeshName()
        try:
            meshcontext = oof2.engine.mesh.meshes[mesh] ## a who object
        except KeyError:
            return plist
        meshcontext.reserve()
        try:
            prob_dict = meshcontext.getObject().subproblems
            plist = prob_dict.keys()
        finally:
            meshcontext.cancel_reservation()
        return plist
    def newProblemSelection(self, gtkobj, name):
        # Only do the deselect if the last selection exists and has
        # a different name than the new selection.
        if self.last_selection:
            if self.last_selection==name:
                return
        if name:
            self.last_selection = name
        else:
            self.last_selection = None
        self.sensitize()


    def show(self):
        debug.mainthreadTest()
        self.gtk.show_all()
        self.update()

    
    
    ########################

    def currentMSName(self):
        path = labeltree.makePath(self.currentFullMeshName())
        return path[0]
    def currentMS(self):
        msname = self.currentMSName()
        if msname:
            return microstructure.microStructures[msname].getObject()

    def currentSkeletonName(self):
        path = labeltree.makePath(self.currentFullMeshName())
        return path[1]

    def currentSkeleton(self):
        ms = self.currentMS()
        meshname = self.currentSkeletonName()
        if meshname:
            return skeletoncontext.getSkeleton(ms, meshname)

    def currentFullMeshName(self):
        path = labeltree.makePath(self.meshwidget.get_value())
        return  path[0]+ ':' + path[1] + ':' + path[2] ## self.meshwidget.get_value()
    def currentMeshName(self):
        path = labeltree.makePath(self.currentFullMeshName())
        return path[2]
    def currentMeshContext(self):
        try:
            return oof2.engine.mesh.meshes[self.currentFullMeshName()]
        except KeyError:
            return None
    def currentMesh(self):
        try:
            return oof2.engine.mesh.meshes[self.currentFullMeshName()].getObject()
        except KeyError:
            return None
    
            
    def sensitize(self):
        debug.mainthreadTest()
        msok = self.currentMS() is not None
        skelok = msok and self.currentSkeleton() is not None
        meshok = skelok and self.currentMesh() is not None and \
                 not self.currentMeshContext().query_reservation()
        self.regionpage.draw_grouplist()

        self.newsubproblem.set_sensitive(meshok)
        self.regionpage.setButton.set_sensitive(meshok)
        ## self.fieldtable.set_sensitive(meshok)
        ## self.initlist.set_sensitive(meshok)
        ## self.initbutton.set_sensitive(meshok and self.currentInit is not None)

        ## self.copybutton.set_sensitive(meshok)
        
    def update(self):
        self.set_state(self.currentFullMeshName())

    def set_state(self, meshpath):
        debug.mainthreadTest()
        path = labeltree.makePath(meshpath)
        self.meshwidget.set_value(path)
        ## see fieldPage for further elaborations

    def newSkeleton(self, skeletonname): # switchboard ("new who", "Skeleton")
        self.sensitize()

    def newMesh(self, meshname):        # switchboard ("new who", "Mesh")
        self.set_state(meshname)
        self.sensitize()
        
    def meshCB(self, meshpath):         # WhoWidget callback
        self.set_state(meshpath)
        self.sensitize()

    def removeMesh(self, path):    # switchboard ("remove who", "Mesh")
        self.update()
        self.sensitize()

    def reservationChanged(self, whoobj):
        # switchboard "made reservation", "cancelled reservation"
        if  self.currentFullMeshName() == whoobj.path():
            self.sensitize()

    def buttonpressCB(self, gtkwidget, gdkevent):
        if gdkevent.type == GDK._2BUTTON_PRESS:
            # A double-click generates a "select-row" signal, a
            # "button_press_event" with event type 2BUTTON_PRESS, and
            # then another "select-row" signal.  Although it would be
            # nice to call initbuttonCB() here directly upon receipt of
            # the double click, it doesn't work because initbuttonCB()
            # brings up a modal dialog box, and the next signal to be
            # processed is the second "select-row" for this gtk.CList,
            # which isn't in the modal dialog.  That hangs X (at least
            # it hangs XFree86 on OS X).  It may be a bug in gtk+ or
            # X.  In any case, the solution is to delay raising the
            # modal dialog until the second "select-row" is issued.
            self.doubleclicked = 1

    

    
    

#############

spp = SubProblemPage()
