# -*- python -*-
# $RCSfile: pixelselect.py,v $
# $Revision: 1.16 $
# $Author: langer $
# $Date: 2005/05/13 18:44:06 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from oof2.SWIG.engine import pixelselectioncouriere
from oof2.common import pixelselection
from oof2.common import pixelselectionmod
from oof2.common import registeredclass
from oof2.common.IO import parameter
from oof2.common.IO import whoville
from oof2.engine import skeletoncontext

ElementSelection = pixelselectioncouriere.ElementSelection
class SelectPixelsInElement(pixelselectionmod.SelectionModifier):
    def __init__(self, skeleton):
        self.skeleton = skeleton
    def __call__(self, ms, selection):
        skel = skeletoncontext.skeletonContexts[self.skeleton].getObject()
        selection.start()
        for el in skel.elements:
            if el.selected:
                selection.select(ElementSelection(ms, el))

registeredclass.Registration(
    'Select Element Pixels',
    registeredclass=pixelselectionmod.SelectionModifier,
    subclass=SelectPixelsInElement,
    ordering=100,
    params=[whoville.WhoParameter('skeleton', skeletoncontext.skeletonContexts,
                                  tip=parameter.emptyTipString)
    ],
    tip="Select the pixels lying under the selected elements.",
    discussion="""<para>
    Select all pixels that intersect the currently selected &skel; elements.
    </para>""")

SegmentSelection = pixelselectioncouriere.SegmentSelection
class SelectPixelsUnderSegment(pixelselectionmod.SelectionModifier):
    def __init__(self, skeleton):
        self.skeleton = skeleton
    def __call__(self, ms, selection):
        skel = skeletoncontext.skeletonContexts[self.skeleton].getObject()
        selection.start()        
        for segment in skel.segments.values():
            if segment.selected:
                n0 = segment.nodes()[0].position()
                n1 = segment.nodes()[1].position()
                selection.select(SegmentSelection(ms, n0, n1))

registeredclass.Registration(
    'Select Segment Pixels',
    registeredclass=pixelselectionmod.SelectionModifier,
    subclass=SelectPixelsUnderSegment,
    ordering=105,
    params=[whoville.WhoParameter('skeleton', skeletoncontext.skeletonContexts,
                                  tip=parameter.emptyTipString)
            ],
    tip="Select the pixels lying under the selected skeleton segments",
    discussion="""<para>
    
    Select all pixels that intersect the currently selected &skel; segments.

    </para>""")
